#!/bin/bash

. ./config

PKG="$1"
ARCH="$2"
CURDIR=$(pwd)

exit_if_error() {
    local status=$?

    if [ $status != 0 ]; then
        exit $status
    fi
}

checkinstalled () {
    while [ -n "$1" ]; do
        dpkg -l "$1" |grep -q "^ii" || {
            echo "$1 is not installed."
            exit 1
        }
        shift
    done
}

# find full version string: version <binarypackage>
version () {
	rmadison -a ${ARCH} ${1} | grep -w ${SUITE} | cut -d'|' -f2 | tr -d ' ' | cut -d'+' -f1 |sed 's/1://'|tail -n 1
}

srcver () {
	local x=$(apt-cache showsrc $1 |grep ^Version: |sed 's/^Version: //' |sort -bru |head -1)
	echo $(echo $x |sed 's/\(.*\)\(-[0-9]*$\)/\1/' |sed 's/1://')
}

# find upstream version string: upstreamversion <binarypackage>
upstreamversion () {
	echo $(version $1 |sed 's/\(.*\)\(-[0-9]*$\)/\1/')
}

# find debian revision string: debianrevision <binarypackage>
debianrevision () {
	echo $(version $1 |sed 's/\(.*-\)\([0-9]*$\)/\2/')
}

# find sourcepackage: sourcepackage <binarypackage>
sourcepackage () {
	local x=$(grep-status -PX $1 -s Source: -n)
	local y=$1
	echo ${x:=$y}
}

# find source directory: sourcedir <sourcepackage>
sourcedir () {
	echo $1-$(srcver $1)
}

# find source directory: sourcedir_v <sourcepackage>
sourcedir_v () {
        local _a=`sourcedir $(srcver $1)`
        local _b=`echo ${_a} | cut -d'-' -f1`
        echo $1-${_b}
}

# applies patches 
apply_patch () {
	patch -p1 < ../../patches/$1
}

# make cross ${ARCH} package: make-cross-package <binarypackage>
make-cross-package () {
    # make cross package
    apt-cross -q --arch ${ARCH} --suite ${SUITE} --mirror ${MIRROR} --build --force $1
    exit_if_error
}

sanitycheck () {
    # sanity check build environment
    echo -n "Sanity check: "

    checkinstalled coreutils
    checkinstalled patchutils
    checkinstalled sudo
    checkinstalled fakeroot
    checkinstalled time
    checkinstalled build-essential
    checkinstalled autogen
    checkinstalled autoconf
    checkinstalled automake1.9
    checkinstalled libtool
    checkinstalled gawk
    checkinstalled bison
    checkinstalled flex
    checkinstalled realpath

    checkinstalled dpkg-cross

    checkinstalled pbuilder
    checkinstalled devscripts
    checkinstalled dpatch
    checkinstalled dh-make
    checkinstalled debhelper

    # for libs
    checkinstalled libterm-progressbar-perl
    checkinstalled libcache-apt-perl
    checkinstalled apt-cross

    # for binutils
    checkinstalled dejagnu

    # for gcc
    checkinstalled chrpath
    checkinstalled doxygen
    checkinstalled graphviz

    # for gdb 
    checkinstalled gcj-4.3
    checkinstalled gij
    checkinstalled gobjc-4.3
    checkinstalled libecj-java
    checkinstalled libecj-java-gcj
    checkinstalled libgcj9-dev
    checkinstalled libobjc2
    checkinstalled libreadline5-dev
    checkinstalled type-handling

    # dpkg-cross version check
    if [[ "2.4.0" > $(dpkg-cross --help 2>&1 |awk '{print $3; exit;}') ]]; then
        echo "dpkg-cross is too old."
        exit 1
    fi

    echo "OK."
}

# build and install libraries needed to build the cross toolchain
build-libs () {
    make-cross-package linux-libc-dev

    make-cross-package libgcc1
    make-cross-package libgcc1-dbg
    make-cross-package gcc-4.3-base

    make-cross-package libdb1-compat

    make-cross-package libc6
    make-cross-package libc6-dev

    if [ ${ARCH} = "powerpc" ]; then
        make-cross-package libc6-ppc64
        make-cross-package libc6-dev-ppc64
    fi

    rm *_${ARCH}.deb

    ${ROOTCMD} dpkg -i *-${ARCH}-cross*_all.deb
    exit_if_error
}

# build and install cross binutils
build-binutils () {
	local _upver=$(upstreamversion binutils)
	local _version=$(version binutils)
	local _dir=$(sourcedir binutils)
	local _debarch=${ARCH}

	if [ ! -d binutils-${_version} ]; then
		rm -rf $_dir
		apt-get source binutils
                exit_if_error

		cd $_dir

		echo "${ARCH}" > debian/target
		#// Workarround patches
		# apply_patch binutils-cross-2.17.patch
		# apply_patch binutils-cross-2.18.patch
		# apply_patch binutils-cross-2.18.1cvs20071027.patch
		apply_patch binutils-cross-2.18.1cvs20080103.patch
                exit_if_error
		apply_patch binutils-2.18.1~cvs20080103.armv6-mull-reg-overlap-warn.patch
                exit_if_error

		cd ..
		mv $_dir binutils-${_version}
	fi

	cd binutils-${_version}

	${ROOTCMD} /usr/lib/pbuilder/pbuilder-satisfydepends-classic
        exit_if_error

	TARGET=${ARCH} fakeroot debian/rules binary-cross
        exit_if_error

	#dpkg-genchanges > binutils_${_version}_${debarch}.changes

	cd ..

	$ROOTCMD dpkg -i --force-overwrite binutils-*-linux-gnu*.deb >> dpkg.log
        exit_if_error

	#// Do clean up
	rm -rf *.gz *.bz2 *.tgz *.dsc *.changes *.tar binutils-${_version}/
}

# build cross gcc versions: build-gcc <majorversion>
build-gcc () {
	local _upver=$(upstreamversion gcc-$1)
	local _version=$(version gcc-$1)
	local _dir=$(sourcedir gcc-$1)
	local _vers=$(sourcedir_v gcc-$1)

	if [ ! -d gcc-$1-${_version} ]; then
		rm -rf $_dir
		apt-get source gcc-$1
                exit_if_error

		cd $_vers

		if [ ${1} = "4.3" ]; then
			#apply_patch omit-base.diff
			apply_patch s390-cross-no-biarch.dpatch
                        exit_if_error
			apply_patch gcc-4.3-4.3.2-gcc-suggests-no-mudflap-gomp.patch
                        exit_if_error
		fi
		
		if [ ${1} = "4.4" ]; then
			apply_patch gcc-4.4-update-cross-fixes.patch
                        exit_if_error
		fi
		
	        ${ROOTCMD} /usr/lib/pbuilder/pbuilder-satisfydepends-classic
                exit_if_error

		GCC_TARGET=${ARCH} DEB_CROSS=yes fakeroot debian/rules control
                exit_if_error

		cd ..

		mv $_vers gcc-$1-${_version}
	fi
	

	cd gcc-$1-${_version}
	
	${ROOTCMD} /usr/lib/pbuilder/pbuilder-satisfydepends-classic
	
	DEB_CROSS_NO_BIARCH=yes \
	GCC_TARGET=${ARCH} DEB_CROSS=yes dpkg-buildpackage -us -uc -rfakeroot
        exit_if_error
	
	cd ..

	$ROOTCMD sh -c "ls -1 *.deb | grep -v gobjc | xargs dpkg -i --force-overwrite" >> dpkg.log
        exit_if_error

	#// Do clean up
        rm -rf *.gz *.bz2 *.tgz *.dsc *.changes *.tar gcc-$1-${_version}/
}

build-gcc-4.3 () {
        checkinstalled gcc-4.3
        GCCVER=${GCCVER:="4.3"}
        build-gcc 4.3
}

build-gcc-4.4 () {
        checkinstalled gcc-4.4
        GCCVER=${GCCVER:="4.4"}
        build-gcc 4.4
}

# build and install cross gdb
build-gdb () {
	local _upver=$(upstreamversion gdb)
	local _version=$(version gdb)
	local _dir=$(sourcedir gdb)
	local _debarch=${ARCH}

	if [ ! -d gdb-${_version} ]; then
		rm -rf $_dir
		apt-get source gdb
                exit_if_error

		cd $_dir

		apply_patch gdb-6.8-3-gdbserver.patch
                exit_if_error
		apply_patch gdb-6.8-3-not-install-man.diff
                exit_if_error

		cd ..
		mv $_dir gdb-${_version}
	fi

	cd gdb-${_version}

	${ROOTCMD} /usr/lib/pbuilder/pbuilder-satisfydepends-classic
	GDB_TARGET=${ARCH} dpkg-buildpackage -rfakeroot -uc -us
        exit_if_error

	cd ..

	# Do clean up
	rm -rf *.gz *.bz2 *.dsc *.changes *.tgz *.tar gdb-${_version}/ 
}

usage () {
    echo "Usage: $0 <package> <arch>"
}

if [ $# != 2 ]; then
    usage
    exit 1
fi

if [ "$PKG" != "libs" -a "$PKG" != "binutils" -a "$PKG" != "gdb" -a "$PKG" != "4.3" ]; then
    echo "As for PKG, only libs, binutils, gdb or gcc-4.3 is supported now."
    exit 1
fi

if [ "$ARCH" != "arm" -a "$ARCH" != "armel" -a "$ARCH" != "powerpc" ]; then
    echo "As for ARCH, only arm, armel or powerpc is supported now."
    exit 1
fi

echo "build $PKG for $ARCH"

if [ ! -d $LOGPATH ]; then
        mkdir -p $LOGPATH
fi

sanitycheck

[ -d ${ARCH} ] || mkdir ${ARCH}
cd ${ARCH}

echo "Updating apt-cross cache"
apt-cross -a ${ARCH} -S ${SUITE} -m ${MIRROR} -u
exit_if_error

ret=0

echo "Building for ARCH::${ARCH}" 
case "$PKG" in
    "libs")
        build-libs 2>&1 | tee -a ../$LOGPATH/$HOST-$ARCH-$PKG.log
        ret=${PIPESTATUS[0]}
        ;;
    "binutils")
        build-binutils 2>&1 | tee -a ../$LOGPATH/$HOST-$ARCH-$PKG.log
        ret=${PIPESTATUS[0]}
        ;;
    "gdb")
        build-gdb 2>&1 | tee -a ../$LOGPATH/$HOST-$ARCH-$PKG.log
        ret=${PIPESTATUS[0]}
        ;;
    "4.3")
        build-gcc-${1} 2>&1 | tee -a ../$LOGPATH/$HOST-$ARCH-$PKG.log
        ret=${PIPESTATUS[0]}
        ;;
    "*")
        echo "Unknown options"
        exit 1
        ;;
esac

cd ${CURDIR}

exit $ret
